#!/usr/bin/python

import cStringIO
import datetime
import gzip
import os
import pickle
import shutil
import subprocess
import sys
import time
import urllib2

from debian_bundle import deb822

from launchpadlib.credentials import Credentials
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError

from bzrlib import transport

#import httplib2
#httplib2.debuglevel = 1

base_dir = "/srv/package-import.canonical.com/new/"
if not os.path.exists(base_dir):
    base_dir = "."
packages_dir = os.path.join(base_dir, "packages")
logs_dir = os.path.join(base_dir, "logs")
retry_dir = os.path.join(logs_dir, "retry")
lists_dir = os.path.join(base_dir, "lists")
lp_creds_file = os.path.join(base_dir, "lp_creds.txt")
lp_cache_dir = os.path.join(base_dir, "lp_cache")
last_time_file = os.path.join(base_dir, "last_run")
stop_file = os.path.join(base_dir, "STOP_PLEASE")

f = open(lp_creds_file, "rb")
try:
    creds = Credentials("package-import")
    creds.load(f)
finally:
    f.close()
lp = Launchpad(creds, service_root=EDGE_SERVICE_ROOT, cache=lp_cache_dir)
debian = lp.distributions['ubuntu']
ubuntu = lp.distributions['ubuntu']
d_archive = debian.main_archive
u_archive = ubuntu.main_archive


def update_list(release, component, source_url):
    target_dir = os.path.join(lists_dir, "dists", release, component, "source")
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)
    target_path = os.path.join(target_dir, "Sources.gz")
    target_tsfile = target_path + ".ts"
    ts = None
    if os.path.exists(target_tsfile):
        f = open(target_tsfile)
        try:
            ts = f.read().strip()
        finally:
            f.close()
    req = urllib2.Request(source_url)
    if ts is not None:
        req.add_header('If-Modified-Since', ts)
    opener = urllib2.build_opener()
    print "Trying %s" % source_url
    try:
        resp = opener.open(req)
    except urllib2.HTTPError, e:
        if e.code != 304:
            raise
        print "Not modified"
    else:
        print "modified"
        f = open(target_path, "wb")
        try:
            shutil.copyfileobj(resp, f)
        finally:
            f.close()
        ts = resp.headers.get('Last-Modified')
        f = open(target_tsfile, "wb")
        try:
            f.write("%s\n" % ts)
        finally:
            f.close()

def update_lists():
    for release in ("etch", "lenny", "squeeze", "sid", "experimental"):
        for component in ("main", "contrib", "non-free"):
            source_url = "http://ftp.uk.debian.org/debian/dists/%s/%s/source/Sources.gz" % (release, component)
            update_list(release, component, source_url)
    for release in ("woody", "sarge"):
        for component in ("main", "contrib", "non-free"):
            source_url = "http://archive.debian.org/debian/dists/%s/%s/source/Sources.gz" % (release, component)
            update_list(release, component, source_url)


def pool_base(name):
    if name.startswith("lib"):
        return name[:4]
    return name[0]

def import_package(name):
    failure_file = os.path.join(logs_dir, "failures", name)
    if os.path.exists(failure_file):
        print "Skipping %s, failed\n" % name
        return
    print "Trying %s" % name
    # Create an empty failure to protect against this script
    # dying.
    f = open(failure_file, "wb")
    try:
        f.write('Apparently the supervisor died\n')
    finally:
        f.close()
    proc = subprocess.Popen("./import_package.py %s" % name,
        shell=True, stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT)
    ret = proc.wait()
    if ret != 0:
        output = proc.stdout.read()
        f = open(failure_file, "wb")
        try:
            f.write(output)
        finally:
            f.close()
        print "Importing %s failed:\n%s" % (name, output.encode("ascii", "replace"))
    else:
        os.unlink(failure_file)
        output = proc.stdout.read()
        print "Success %s:\n%s" % (name, output.encode("ascii", "replace"))


def mass_import():
    last_checked = None
    while True:
        if os.path.exists(stop_file):
            print "Stopping as requested."
            return
        last_known_published = None
        if os.path.exists(last_time_file):
            f = open(last_time_file)
            try:
                last_known_published = pickle.load(f)
                # Lose 10 minutes just in case
                last_known_published = \
                        last_known_published - datetime.timedelta(0, 600)
            finally:
                f.close()
        # We want all the new Published records since the last run
        checked = set()
        newest_published = None
        for publication in u_archive.getPublishedSources(status="Published",
                published_since_date=last_known_published.isoformat()):
            if publication.source_package_name in checked:
                continue
            if os.path.exists(stop_file):
                print "Stopping as requested."
                return
            published_time = datetime.datetime.strptime(publication.date_published[:19],
                       "%Y-%m-%dT%H:%M:%S")
            if (last_known_published is None
                    or published_time > last_known_published):
                # We've got a live one
                name = publication.source_package_name
                checked.add(name)
                import_package(name)
                if (newest_published is None
                        or published_time > newest_published):
                    newest_published = published_time
        if newest_published is not None:
            # Save the last publication date we found
            f = open(last_time_file, "wb")
            try:
                pickle.dump(newest_published, f)
            finally:
                f.close()
        if (last_checked is not None
                and len(checked.symmetric_difference(last_checked)) < 1):
            time.sleep(300)
        last_checked = checked
        retries = os.listdir(retry_dir)
        for retry in retries:
            os.unlink(os.path.join(retry_dir, retry))
            import_package(retry)


def full_import(start_letter, end_letter):
    try:
        main_pkgnames = set()
        pkgnames = set()
        url_base = "http://archive.ubuntu.com/ubuntu/dists/karmic/"
        resp = urllib2.urlopen(url_base + "main/source/Sources.gz")
        contents = cStringIO.StringIO(resp.read())
        f = gzip.GzipFile(fileobj=contents)
        for source in deb822.Sources.iter_paragraphs(f):
            main_pkgnames.add(source['Package'])
        #for s in [ubuntu.current_series]:
        #    for publication in u_archive.getPublishedSources(status="Published",
        #            distro_series=s):
        #        if s == ubuntu.current_series:
        #            if publication.component_name == "main":
        #                main_pkgnames.add(publication.source_package_name)
        #        pkgnames.add(publication.source_package_name)
        #for s in debian.series:
        #    for publication in u_archive.getPublishedSources(status="Pending",
        #            distro_series=s):
        #        pkgnames.add(publication.source_package_name)
        i = 0
        for pkgname in main_pkgnames:
            if os.path.exists(stop_file):
                print "Stopping as requested."
                return
            if pkgname[:len(start_letter)] < start_letter or pkgname[:len(end_letter)] > end_letter:
                continue
            if os.path.exists(os.path.join(logs_dir, pkgname)):
                print "Skipping %s, log already present" % pkgname
                continue
            if i % 10 == 0:
                update_lists()
            import_package(pkgname)
            i += 1
    except HTTPError, e:
        print e.content
        raise

start_letter = end_letter = None

if len(sys.argv) > 1:
    if len(sys.argv) != 3:
        print "Specify start and end letters"
        print sys.argv
        sys.exit(1)
    start_letter = sys.argv[1]
    end_letter = sys.argv[2]

#mass_import()
full_import(start_letter, end_letter)
