#!/usr/bin/python

from optparse import OptionParser
import os
import sys

from bzrlib.branch import Branch
from bzrlib.revisionspec import RevisionSpec

base_dir = "/srv/package-import.canonical.com/new/"
logs_dir = os.path.join(base_dir, "logs")
failures_dir = os.path.join(logs_dir, "failures")

parser = OptionParser()
parser.add_option("--revision", "-r", type=str,
	 help="Revision to requeue the failures from")
opts, args = parser.parse_args()

def requeue(package):
    fail_file = os.path.join(failures_dir, package)
    if os.path.exists(fail_file):
	os.remove(fail_file)
	print "%s requeued" % package
	return True
    else:
        print "%s has not failed" % package
	return False

failures = 0

if opts.revision is None:
    if len(args) < 1:
        print "Specify a list of packages to requeue or --revision"
        sys.exit(1)
else:
    b = Branch.open(logs_dir)
    spec = RevisionSpec.from_string(opts.revision)
    rev_id = spec.in_history(b).rev_id
    rev_tree = b.repository.revision_tree(rev_id)
    old_rev_id = rev_tree.get_parent_ids()[0]
    old_rev_tree =  b.repository.revision_tree(old_rev_id)
    changes = rev_tree.changes_from(old_rev_tree)
    for added in changes.added:
        if added[0].startswith("failures"):
             package = added[0][len("failures/"):]
	     if not requeue(package):
                 failures += 1

for package in args:
    if not requeue(package):
        failures += 1

if failures > 0:
    sys.exit(1)

